from bs4 import BeautifulSoup
import re
from openpyxl import Workbook
from openpyxl.utils import get_column_letter
from openpyxl.utils.cell import absolute_coordinate
from openpyxl.styles import PatternFill, Alignment, Font
from openpyxl.formatting.rule import ColorScaleRule, CellIsRule
from openpyxl import load_workbook

Thresholds = [2,1.9,2,1.9,2.4,2.3,1.8,1.9]
def imput_seq():
    Secuencia = open(wdir+"/input/input-seq.txt", "r")
    red_seq = Secuencia.read()
    Secuencia.close

    with open(wdir+"/input/input-seq.txt") as file:
        lineas = [line.rstrip() for line in file]
    lista = []
    for i in lineas:
        if i.startswith(">"):
            lista.append("aquies")
        else:
            lista.append(i)
            
    if lista[0] == "aquies":
        lista.remove("aquies")
    seq = "".join(lista)
    seq = seq.upper()
    imput_seq.seq = seq
    
    no_amino_acidos = ['J','O','U','X']
    for i in no_amino_acidos:
        isit = seq.find(i)
        if isit != -1:
            print ("\nEsta no es una secuencia válida de amino acidoácidos\n")
            print ("\nCaracter no válido en la posición ", isit,".\n") 
            break

def bcepredresults(seq,Thresholds):
    bce_file = wdir+"/bcepred/bcepred.html"
    page = open(bce_file)
    
    soup = BeautifulSoup(page.read(), "html.parser")
    
    
    #Paso 1: Dejar únicamente el ovrlap display (la parte que sí sirve.
    #cambiar los <br> por retornos de linea.
    phase0 = re.sub(r'<u>([A-Z])</u>','Z',str(soup))
    phase1 = re.sub(r'<.+?>','',str(phase0).replace('<tr>','\n'))
    phase2 = re.sub(r'<.+?>','',str(phase1).replace('<br/>','\n'))
    
    #Paso 2: delimitar hasta cual línea se debe cortar
    beginning = phase2.index ("The predicted")
    phase3 = phase2[beginning + 71: ]
    phase4 = re.sub(r'<u>([A-Z])</u>','Z',str(phase3))
    phase5 = re.sub(r'<.+?>','',str(phase4).replace('<tr>','\n'))
    phase6 = re.sub(r'1','\n',str(phase5))
    
    antigen_results = re.sub(r'([0-9])','',str(phase6))
    
    results_list = antigen_results.split("\n")
    
    for i, j in enumerate(results_list):
        if j == "":
            del results_list[i]
    
    #Tomando en cuenta que este programa da un resultado mas complejo (múltiples resultados, resultados 
    #Asegurar que todos los resultados estén en orden
    
        
    while len(results_list) <= 16:
        results_list.append ("N/A")
    
    
    if results_list[2].find("Hydrophilicity") == -1:
        results_list.insert(2, "Hydrophilicity")
        if results_list [3] != "N/A":
            results_list.insert(3, "N/A")
    
    if results_list[4].find("Flexibility") == -1:
        results_list.insert(4, "Flexibility")
        if results_list [5] != "N/A":
            results_list.insert(5, "N/A")
    
    if results_list[6].find("Accessibility") == -1:
        results_list.insert(6, "Accessibility")
        if results_list [7] != "N/A":
            results_list.insert(7, "N/A")
    
    if results_list[8].find("Turns") == -1:
        results_list.insert(8, "Turns")
        if results_list [9] != "N/A":
            results_list.insert(9, "N/A")
    
    if results_list[10].find("Exposed") == -1:
        results_list.insert(10, "Exposed Surface")
        if results_list [11] != "N/A":
            results_list.insert(11, "N/A")
    
    if results_list[12].find("Polarity") == -1:
        results_list.insert(12, "Polarity")
        if results_list [13] != "N/A":
            results_list.insert(13, "N/A")
    
    if results_list[14].find("Antigenic") == -1:
        results_list.insert(14, "Antigenic Propensity")
        if results_list [15] != "N/A":
            results_list.insert(15, "N/A")
    
    while len(results_list) > 16:
        del results_list[16]
    bcepredresults.results_list = results_list
  
    
    
def results2xls(seq):
    seq = imput_seq.seq
    the_matrix = bcepredresults.results_list
    wb = load_workbook(wdir+'/ICE-BIOSEQ-out.xlsx') 
    ws = wb.active
    ws['A2'] = "Secuencia"
    ws['A2'].font = Font(bold=True)
    
    ws["A4"] = "BCEPred"
    ws['A4'].font = Font(bold=True)
    
    tasks = ["Hydrophilicity", "Flexibility", "Accesibility", "Turns", "Exposed surface", "Polarity", "Antigenic propencity"]
    colors = ["00b0f0", "92d050", "ff0000", "ffff00", "f4b084", "ffd966", "c9c9c9"]
    
    coor = 5
    for t in (tasks):
        ws.cell(row=coor, column=1).value = t
        coor = coor + 1
    
    coor = 5
    for c in colors:
        ws.cell(row=coor, column=1).fill = PatternFill(fgColor= c, fill_type = "solid")
        coor = coor+1
    
    #Escribir la secuencia base
    for x, y in enumerate(seq):
        ws.cell(row= 2, column= x+2).value = y
        
    #Escribir la secuencia en cada una de las tasks    
    for i, h in enumerate(tasks):
        le_row = i + 5
        for x, y in enumerate(seq):
            ws.cell(row= le_row, column= x+2).value = y
    
    #colorear los resultados de hydrophilicity     
    if the_matrix[3] != "N/A":
        for i, h in enumerate (seq):
            dummy = (the_matrix[3])
            dummer = dummy[i]
            poc = "Z"
            if poc == dummer:
                ws.cell(row=5, column=2+i).fill = PatternFill(fgColor= colors[0], fill_type = "solid")
    
    #colorear los resultados de Flexibility   
    if the_matrix[5] != "N/A":
        for i, h in enumerate (seq):
            dummy = (the_matrix[5])
            dummer = dummy[i]
            poc = "Z"
            if poc == dummer:
                ws.cell(row=6, column=2+i).fill = PatternFill(fgColor= colors[1], fill_type = "solid")
            
    #colorear los resultados de Accesibility
    if the_matrix[7] != "N/A":
        for i, h in enumerate (seq):
            dummy = (the_matrix[7])
            dummer = dummy[i]
            poc = "Z"
            if poc == dummer:
                ws.cell(row=7, column=2+i).fill = PatternFill(fgColor= colors[2], fill_type = "solid")

    #colorear los resultados de Turns
    if the_matrix[9] != "N/A":
        for i, h in enumerate (seq):
            dummy = (the_matrix[9])
            dummer = dummy[i]
            poc = "Z"
            if poc == dummer:
                ws.cell(row=8, column=2+i).fill = PatternFill(fgColor= colors[3], fill_type = "solid")
            
    #colorear los resultados de Exposed surface
    if the_matrix[11] != "N/A":       
        for i, h in enumerate (seq):
            dummy = (the_matrix[11])
            dummer = dummy[i]
            poc = "Z"
            if poc == dummer:
                ws.cell(row=9, column=2+i).fill = PatternFill(fgColor= colors[4], fill_type = "solid")
    
    #colorear los resultados de Polarity
    if the_matrix[13] != "N/A":        
        for i, h in enumerate (seq):
            dummy = (the_matrix[13])
            dummer = dummy[i]
            poc = "Z"
            if poc == dummer:
                ws.cell(row=10, column=2+i).fill = PatternFill(fgColor= colors[5], fill_type = "solid")    
    
    #colorear los resultados de Antigenic propencity
    if the_matrix[15] != "N/A":        
        for i, h in enumerate (seq):
            dummy = (the_matrix[15])
            dummer = dummy[i]
            poc = "Z"
            if poc == dummer:
                ws.cell(row=11, column=2+i).fill = PatternFill(fgColor= colors[6], fill_type = "solid")  
    wb.save(wdir+'/ICE-BIOSEQ-out.xlsx')

def start(wodir):
    global wdir
    wdir = wodir
    imput_seq()
    bcepredresults(imput_seq.seq, Thresholds)
    results2xls(imput_seq.seq)


from bs4 import BeautifulSoup
import re
import pandas as pd
from openpyxl import Workbook
from openpyxl.utils import get_column_letter
from openpyxl.utils.cell import absolute_coordinate
from openpyxl.styles import PatternFill, Alignment, Font
from openpyxl.formatting.rule import ColorScaleRule, CellIsRule
from openpyxl import load_workbook

def input_seq ():
    Secuencia = open(wdir+"/input/input-seq.txt", "r")
    red_seq = Secuencia.read()
    Secuencia.close

    with open(wdir+"/input/input-seq.txt") as file:
        lineas = [line.rstrip() for line in file]
    lista = []
    for i in lineas:
        if i.startswith(">"):
            lista.append("aquies")
        else:
            lista.append(i)
            
    if lista[0] == "aquies":
        lista.remove("aquies")
    seq = "".join(lista)
    seq = seq.upper()
    input_seq.seq = seq
    
    no_amino_acidos = ['J','O','U','X']
    for i in no_amino_acidos:
        isit = seq.find(i)
        if isit != -1:
            print ("\nEsta no es una secuencia válida de amino acidoácidos\n")
            print ("\nCaracter no válido en la posición ", isit,".\n") 
            break

def BEPI_results(seq):
    data = pd.read_csv(wdir+"/bepipred/bepi.csv")
    scores = list(data.EpitopeProbability)
    BEPI_results.results_list = scores
    
def results2xls(seq):
    seq = input_seq.seq
    the_matrix = BEPI_results.results_list
    wb = load_workbook(wdir+'/ICE-BIOSEQ-out.xlsx') 
    ws = wb.active
    
    ws["A15"] = "BEPIPred"
    ws["A15"].font = Font(bold=True)
    ws["A15"].fill = PatternFill(fgColor= "00FF00", fill_type = "solid")
    
    #Esribir la secuencia
    for x, y in enumerate(seq):
        ws.cell(row= 15, column= x+2).value = y
    
    #Colorear los resultados
    for i, h in enumerate (seq):
        dummy = the_matrix[i]
        poc = float(0.5)
        if dummy >= poc:
            ws.cell(row=15, column=2+i).fill = PatternFill(fgColor= "00FF00", fill_type = "solid")
    
    wb.save(wdir+"/ICE-BIOSEQ-out.xlsx")

def start(wodir):
    global wdir
    wdir = wodir
    input_seq()
    BEPI_results(input_seq.seq)
    results2xls(input_seq.seq)



from . import bce
from . import abc
from . import bepi
from . import iedb1
from . import iedb2

#exec(open("abc.py").read())
#exec(open("bce.py").read())
#exec(open("bepi.py").read())
#exec(open("iedb1.py").read())
#exec(open("iedb2.py").read())


from bs4 import BeautifulSoup
import re
from openpyxl import Workbook
from openpyxl.utils import get_column_letter
from openpyxl.utils.cell import absolute_coordinate
from openpyxl.styles import PatternFill, Alignment, Font
from openpyxl.formatting.rule import ColorScaleRule, CellIsRule
from openpyxl import load_workbook



def imput_seq():
    Secuencia = open(wdir+"/input/input-seq.txt", "r")
    red_seq = Secuencia.read()
    Secuencia.close

    with open(wdir+"/input/input-seq.txt") as file:
        lineas = [line.rstrip() for line in file]
    lista = []
    for i in lineas:
        if i.startswith(">"):
            lista.append("aquies")
        else:
            lista.append(i)
            
    if lista[0] == "aquies":
        lista.remove("aquies")
    seq = "".join(lista)
    seq = seq.upper()
    imput_seq.seq = seq
    
    no_amino_acidos = ['J','O','U','X']
    for i in no_amino_acidos:
        isit = seq.find(i)
        if isit != -1:
            print ("\nEsta no es una secuencia válida de amino acidoácidos\n")
            print ("\nCaracter no válido en la posición ", isit,".\n") 
            break

def iedb1_results(seq):
    iedb_url = wdir+"/iedb1/iedb-mhci.html"
    iedb_page = open(iedb_url)
        
    soup = BeautifulSoup(iedb_page.read(), "html.parser")
    #Dejar únicamente la tabla de los resultados.
    #La salida de BS4 debe ser convertida a string antes de trabajarla como tal
    sopa = str(soup)
    #Cambir tabuladores por espacios
    sopa2 = re.sub(r'\t',' ',str(sopa))
    #Hacer una lista con cada "renglón"
    sopa_table = sopa2.split("\n")
    #Lista donde cada renglón se divide como lista
    sopa_table2 = []
    for i in sopa_table:
        sopa_table2.append(i.split())
    #Crear una lista únicamente con los resultados.
    sopa_table3 = []
    for i in sopa_table2:
        if len(i) == 8:
            sopa_table3.append(i)
    del sopa_table3[0]
    del sopa_table3[0]
    #Hacer una lista con los datos relevantes para el análisis antigénicos
    chosen_ones = []
    for i in sopa_table3:
        if float(i[6]) >= 0.85:
            chosen_ones.append(i)
        else:
            break
    iedb1_results.results_list = chosen_ones
     
def results2xls(seq):
    seq = imput_seq.seq
    the_matrix = iedb1_results.results_list
    wb = load_workbook(wdir+'/ICE-BIOSEQ-out.xlsx') 
    ws = wb.active
    
    ws["A17"] = "IEDB MHCI"
    ws["A17"].font = Font(bold=True)
    ws["A17"].fill = PatternFill(fgColor= "5B9BD5", fill_type = "solid")
    
    #Esribir la secuencia
    for x, y in enumerate(seq):
        ws.cell(row= 17, column= x+2).value = y
    
    for i in the_matrix:
        start = int(i[2])
        end = int(i[3])
        while start <= end:
            ws.cell(row=17, column=1+start).fill = PatternFill(fgColor= "5B9BD5", fill_type = "solid")
            start = start+1
        
    wb.save(wdir+"/ICE-BIOSEQ-out.xlsx")
    
def start(wodir):
    global wdir
    wdir = wodir
    imput_seq()
    iedb1_results(imput_seq.seq)
    results2xls(imput_seq.seq)


from bs4 import BeautifulSoup
import re 
from openpyxl import Workbook
from openpyxl.utils import get_column_letter
from openpyxl.utils.cell import absolute_coordinate
from openpyxl.styles import PatternFill, Alignment, Font
from openpyxl.formatting.rule import ColorScaleRule, CellIsRule
from openpyxl import load_workbook

def input_seq ():
    secuencia = open(wdir+"/input/input-seq.txt", "r")
    Secuencia = open(wdir+"/input/input-seq.txt", "r")
    red_seq = Secuencia.read()
    Secuencia.close

    with open(wdir+"/input/input-seq.txt") as file:
        lineas = [line.rstrip() for line in file]
    lista = []
    for i in lineas:
        if i.startswith(">"):
            lista.append("aquies")
        else:
            lista.append(i)
            
    if lista[0] == "aquies":
        lista.remove("aquies")
    seq = "".join(lista)
    seq = seq.upper()
    input_seq.seq = seq
    
    no_amino_acidos = ['J','O','U','X']
    for i in no_amino_acidos:
        isit = seq.find(i)
        if isit != -1:
            print ("\nEsta no es una secuencia válida de amino acidoácidos\n")
            print ("\nCaracter no válido en la posición ", isit,".\n") 
            break


def abcpredresults(seq):
    abc_url = wdir+"/abcpred/abcpred_res.html"
    abc_page = open(abc_url)
    
    soup = BeautifulSoup(abc_page.read(), "html.parser")
    
    #Paso 1: Dejar únicamente el ovrlap display (la parte que sí sirve.
    #cambiar los <br> por retornos de linea.
    phase1 = stringTable_02 = re.sub(r'<.+?>','',str(soup).replace('<br/>','\n'))
    #Paso 2: delimitar hasta cual línea se debe cortar
    beginning = phase1.index ("DISPLAY")
    phase2 = phase1[beginning + 7: ]
    phase2 = re.sub(r'([A-Z])','Z',str(phase2))
    #Paso 3: Eliminar la secuencia, dejando solo los resultados
    phase3 = phase2.split("\n")
    del phase3[0]
    phase3.reverse()
    results_list = ' '.join(phase3).split()
    abcpredresults.results_list = results_list
    
def results2xls(seq):
    seq = input_seq.seq
    the_matrix = abcpredresults.results_list
    wb = load_workbook(wdir+'/ICE-BIOSEQ-out.xlsx') 
    ws = wb.active
    
    ws["A13"] = "ABCPred"
    ws["A13"].font = Font(bold=True, color = "FFFFFF")
    ws["A13"].fill = PatternFill(fgColor= "7030A0", fill_type = "solid")
    
    #Esribir la secuencia
    for x, y in enumerate(seq):
        ws.cell(row= 13, column= x+2).value = y
        
    #Colorear el primer resultados
    hex_base = int(6303950)
    for n in the_matrix:
        for i, h in enumerate (seq):
            dummy = n
            dummer = dummy[i]
            poc = "Z"
            if poc == dummer:
                if hex_base < int(0x10000):
                    color = hex(7352480)
                else:
                    color = hex(hex_base)
                painting = color[2:]
                ws.cell(row=13, column=2+i).fill = PatternFill(fgColor= painting, fill_type = "solid")
                ws.cell(row=13, column=2+i).font = Font(color = "FFFFFF")
        hex_base = hex_base - int(1572888)

    wb.save(wdir+"/ICE-BIOSEQ-out.xlsx")

def start(wodir):
    global wdir
    wdir = wodir
    input_seq()
    abcpredresults(input_seq.seq)
    results2xls(input_seq.seq)


# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['optibioseq']

package_data = \
{'': ['*']}

install_requires = \
['bs4>=0.0.1,<0.0.2',
 'numpy>=1.22.2,<2.0.0',
 'openpyxl>=3.0.9,<4.0.0',
 'pandas>=1.4.0,<2.0.0']

setup_kwargs = {
    'name': 'optibioseq',
    'version': '0.1.3',
    'description': '',
    'long_description': None,
    'author': 'Iván Corona Guerrero',
    'author_email': 'ivan.corona@uaq.mx> y Dr. Francisco Solis Munoz <siscomagma@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
